import setuptools

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name="django-subjective-sort",
    version="1.0.1",
    author="Michael Prather",
    author_email="michael@krit.com",
    description="Allows items of a collection to be manually sorted"
                "(such as for use with drag-and-drop).",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/builtbykrit/django-subjective-sort",
    project_urls={
        "Bug Tracker": "https://github.com/builtbykrit/django-subjective-sort/issues",  # noqa: E501
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Framework :: Django :: 4.1",
        "Framework :: Django :: 4.0",
        "Framework :: Django :: 3.2"
    ],
    package_dir={"": "src"},
    packages=setuptools.find_packages(where="src"),
    python_requires=">=3.9",
)



import math
from typing import List

from django.db import models


class Sortable(models.Model):
    """
    Makes objects manually sortable
    """
    position = models.IntegerField(blank=True, null=True)

    class Meta:
        abstract = True

    @staticmethod
    def sort_by_position(sortables: List['Sortable']) -> None:
        """
        Sorts a list of sortables by position
        """
        sortables.sort(key=lambda s: s.position or math.inf)

    def reposition(self,
                   peers: List['Sortable'],
                   position: int = None) -> List['Sortable']:
        """
        Updates manual list sorting based on a one-indexed position.
        Returns a filtered list of peers that were changed.
        """
        max_position = len(peers) + 1
        if position and position < 1:
            raise ValueError(
                'Position must be at least one.'
            )

        if position and position > max_position:
            position = max_position

        self.position = position
        self.sort_by_position(peers)

        affected_peers = []
        for index, sortable in enumerate(peers):
            index_position = index + 1

            if not position:
                if not sortable.position:
                    continue
                # if this one has a position, retain it
                sortable.position = index_position

            elif not sortable.position:
                if position <= index_position:
                    continue
                # if the position is gt than the indexed position,
                # we have to declare this one's position
                sortable.position = index_position

            elif index_position < position:
                sortable.position = index_position

            else:
                sortable.position = index_position + 1

            affected_peers.append(sortable)

        return affected_peers
